package com.immomo.cosmos;

import com.alibaba.fastjson.JSON;
import com.immomo.cosmos.bean.CAImg;
import com.immomo.cosmos.bean.CAText;
import com.immomo.cosmos.bean.CAVideo;
import okhttp3.FormBody;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.digest.DigestUtils;

import javax.crypto.Cipher;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @description:
 * @author: wang.jianwen
 * @create: 2020-04-24 17:31
 **/
public class CheckAuthUtils {

    private static final OkHttpClient client = new OkHttpClient()
            .newBuilder()
            .readTimeout(2, TimeUnit.SECONDS)
            .connectTimeout(2, TimeUnit.SECONDS)
            .writeTimeout(2, TimeUnit.SECONDS)
            .build();

    public static String checkText(String appKey, String appId,
                               String timestampMillStr, String category, Map<String, String[]> detail, String uniqid) throws IOException {
        String content = JSON.toJSONString(new CAText(timestampMillStr, category, detail, uniqid));
        return getRes(content, appKey, appId);
    }

    public static String checkImg(String appKey, String appId,
                                  String timestampMillStr, String category,
                                  Map<String, String[]> detail, String uniqid, String redirect) throws IOException {
        String content = JSON.toJSONString(new CAImg(timestampMillStr, category, detail, uniqid, redirect));
        return getRes(content, appKey, appId);
    }

    public static String checkVideo(String appKey, String appId,
                                  String timestampMillStr, String category,
                                  Map<String, String[]> detail, String uniqid, String redirect, String source) throws IOException {
        String content = JSON.toJSONString(new CAVideo(timestampMillStr, category, detail, uniqid, redirect, source));
        return getRes(content, appKey, appId);
    }

    public static String checkAudio(String appKey, String appId,
                                    String timestampMillStr, String category,
                                    Map<String, String[]> detail, String uniqid, String redirect) throws IOException {
        return checkImg(appKey, appId, timestampMillStr, category, detail, uniqid, redirect);
    }

    private static String getRes(String content, String appKey, String appId) throws IOException {
        String encryptRes = encrypt(content, appKey);
        Response resp = sendRequest(appId, encryptRes);
        String body = resp.body().string();
        Integer ec = JSON.parseObject(body).getInteger("ec");
        return ec == 0
                ? body
                : body+"\trequest fail appId="+appId+"\tappKey="+appKey+"\tcontent="+content;
    }

    private static byte[] getIV(String appKey) {
        return DigestUtils.md5Hex(appKey.getBytes(StandardCharsets.UTF_8))
                .substring(0, 16).getBytes();
    }

    private static String encrypt(String strToEncrypt, String appKey) {
        try {
            IvParameterSpec ivspec = new IvParameterSpec(getIV(appKey));
            SecretKeySpec secretKey = new SecretKeySpec(appKey.getBytes(StandardCharsets.UTF_8), "AES");
            Cipher cipher = Cipher.getInstance("AES/CBC/PKCS5Padding");
            cipher.init(Cipher.ENCRYPT_MODE, secretKey, ivspec);
            byte[] doFinal = cipher.doFinal(strToEncrypt.getBytes(StandardCharsets.UTF_8));
            Base64 base64 = new Base64();
            byte[] e1 = base64.encode(doFinal);
            return base64.encodeToString(e1);
        } catch (Exception e) {
            System.out.println("Error while encrypting: " + e.toString());
        }
        return null;
    }


    public static String decrypt(String strToDecrypt, String appKey) {
        try {
            IvParameterSpec ivspec = new IvParameterSpec(getIV(appKey));
            SecretKeySpec secretKey = new SecretKeySpec(appKey.getBytes(StandardCharsets.UTF_8), "AES");
            Cipher cipher = Cipher.getInstance("AES/CBC/PKCS5PADDING");
            cipher.init(Cipher.DECRYPT_MODE, secretKey, ivspec);
            Base64 base64 = new Base64();
            byte[] d1 = base64.decode(strToDecrypt);
            byte[] d2 = base64.decode(d1);
            byte[] aFinal = cipher.doFinal(d2);
            return new String(aFinal, Charset.defaultCharset());
        } catch (Exception e) {
            System.out.println("Error while decrypting: " + e.toString());
        }
        return null;
    }

    private static Response sendRequest(String appId, String content) throws IOException {
        FormBody body = new FormBody.Builder()
                .add("appid", appId)
                .add("sign", DigestUtils.sha1Hex(appId + content))
                .add("content", content)
                .build();

        Request request = new Request.Builder()
                .url("https://eco.immomo.com/api/check/send")
                .addHeader("Content-Type", "application/x-www-form-urlencoded")
                .post(body)
                .build();
        return client.newCall(request).execute();
    }
}
